/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.business;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.zip.Deflater;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean;
import org.ow2.petals.tools.webconsole.util.FilePermissionHelper;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.ZipHelper;

/**
 * @author aruffie
 * 
 */
public class InstallSharedLibraryBBean {

    public URL storeSharedLibrary(final IFileUploadPart file, final HttpServletRequest request)
            throws IOException, URISyntaxException {
        File repo = GeneralHelper.getRepositoryFile(request);
        File repoAttachement = new File(new URI(repo.toURI() + "/shared-libraries"));
        if (!repoAttachement.exists()) {
            repoAttachement.mkdir();
        }
        final String filePath = repoAttachement.getAbsolutePath() + File.separator
                + file.getFileName();
        OutputStream out = null;
        InputStream in = null;
        final File artifact = new File(filePath);
        try {
            out = new FileOutputStream(artifact);
            in = file.getFileInputStream();
            byte[] data = new byte[1024];
            while ((in.read(data)) > -1) {
                out.write(data);
            }
        } finally {
            out.flush();
            out.close();
            in.close();
        }
        
        configureArtifact(artifact);
        
        // Create tempory file for override opensuit security
        final File securityArtifact = new File(artifact.getPath()+".res");
        if(!securityArtifact.exists()){
            securityArtifact.createNewFile();
        }
        
        final URL httpUrl = GeneralHelper.artifactFinder(request, repo, repoAttachement, file);
        return httpUrl;
    }

    private void configureArtifact(final File artifact) throws IOException {

        final File directory = new File(artifact.getAbsolutePath().substring(0,
                artifact.getAbsolutePath().indexOf(".zip")));
        ZipHelper.decompress(artifact.getAbsolutePath(), directory.getAbsolutePath(), false);
        for (final File file : directory.listFiles()) {
            FilePermissionHelper.setRights(file, new int[] { 7, 7, 7 }, false);
        }
        artifact.delete();
        ZipHelper.compress(directory.listFiles(), artifact, Deflater.BEST_SPEED);
        FilePermissionHelper.setRights(artifact, new int[] { 7, 7, 7 }, false);
        directory.delete();
    }

    public void beginInstall(final HttpServletRequest request,
            final ManagementUIBean managementUIBean, final String stringURL, final String serverName)
            throws PetalsServiceException, IOException, URISyntaxException {
        
        if(serverName == null || serverName.equals("")){
            throw new PetalsServiceException("Server name must no be null or empty in order to begin shared library installation");
        }
        if(stringURL == null || stringURL.equals("")){
            throw new PetalsServiceException("Url must no be null or empty in order to begin shared library installation");
        }
        
        final URL url = new URL(stringURL);
        ServerTO serverTO = null;
        for (ServerTO server : managementUIBean.getDomain().getServers()) {
            if (server.getName().equals(serverName)) {
                serverTO = server;
            }
        }
        ManagementService service = null;
        try {
            service = (ManagementService) ManagementServiceFactory.getInstance().getService(
                    serverTO.getHost(), Integer.parseInt(serverTO.getJmxPort()),
                    serverTO.getJmxLogin(), serverTO.getJmxPassword());
        } catch (NumberFormatException e) {
            throw new PetalsServiceException(e);
        }
        service.installSharedLibrary(url);

        final File slTree = new File(new URI(GeneralHelper.getRepositoryFile(request).toURI()
                + "/shared-libraries"));
        if (slTree != null && slTree.exists()) {
            GeneralHelper.deleteDirectoryContent(slTree);
        }
    }

}
