/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.business;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;

import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.services.rmiinjector.RmiInjectorService;
import org.ow2.petals.tools.webconsole.services.rmiinjector.RmiInjectorServiceFactory;
import org.ow2.petals.tools.webconsole.services.rmiinjector.model.Property;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.LogLevel;
import org.ow2.petals.tools.webconsole.util.Logger;
import org.ow2.petals.tools.webconsole.util.PetalsConsoleException;

/**
 * @author aruffie
 * 
 */
public class RMIInjectorBBean {

    private static final String RMI_COMPONENT_NAME = "petals-se-rmi";

    private RmiInjectorService service = null;

    private List<MessageExchange> exchanges;

    public RMIInjectorBBean(final ServerTO rmiServer) throws PetalsServiceException {
        service = (RmiInjectorService) RmiInjectorServiceFactory.getInstance().getService();

        /*
         * Create the connexion properties map for rmi
         */
        Map<String, String> connexionPropertiesMap = createConnexionPropertiesMap(rmiServer);

        // Remoting dialoguer hasn't been initialize, do it
        if (service.getRemoteComponentContext() == null) {
            service.initialize(connexionPropertiesMap);
        }

        exchanges = new ArrayList<MessageExchange>();

    }

    public MessageExchange send(final List<File> attachments, final String mep,
            final QName operation, final Long synchronousSend, final QName _interface,
            final QName service, final String endpoint, final List<Property> mEXProperties,
            final List<Property> normalizedProperties, final String content)
            throws PetalsServiceException {

        /*
         * Perform the injection
         */
        MessageExchange exchange = null;
        exchange = this.service.injectMessage(_interface, normalizedProperties, service, endpoint,
                operation, mep, mEXProperties, content, attachments, synchronousSend);

        if (this.exchanges == null) {
            this.exchanges = new ArrayList<MessageExchange>();
        }
        exchanges.add(exchange);
        storeAllReceivedAttachments(exchange);
        return exchange;
    }

    // TODO: voir si vraiment necessaire
    private void storeAllReceivedAttachments(MessageExchange exchange)
            throws PetalsServiceException {
        NormalizedMessage nm = null;
        if (exchange.getMessage("OUT") != null) {
            nm = exchange.getMessage("OUT");
        } else {
            nm = exchange.getFault();
        }
        if (nm != null) {
            final Set<?> set = nm.getAttachmentNames();
            final Iterator<?> it = set.iterator();
            File repoAttachment = null;
            try {
                repoAttachment = new File(GeneralHelper.getRepositoryPath() + "/attachements");
            } catch (final IOException e) {
                throw new PetalsServiceException(
                        "Impossible to create tempory repository attachment");
            }
            while (it.hasNext()) {
                final String attachmentId = (String) it.next();
                final DataHandler handler = nm.getAttachment(attachmentId);
                final File attachment = new File(repoAttachment.getPath() + "/" + handler.getName());
                if (!attachment.exists()) {
                    InputStream in = null;
                    try {
                        attachment.createNewFile();
                        in = new FileInputStream(attachment);
                        handler.getOutputStream();
                    } catch (final IOException e) {
                        throw new PetalsServiceException(
                                "Impossible to store the returned attachment: " + handler.getName());
                    } finally {
                        if (in != null) {
                            try {
                                in.close();
                            } catch (final IOException e) {
                                throw new PetalsServiceException(e);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Create the connexion properties map, for RMI Connexion
     * 
     * @param rmiServer
     * @return
     * @throws PetalsConsoleException 
     */
    private Map<String, String> createConnexionPropertiesMap(final ServerTO rmiServer) {
        if (rmiServer != null) {
            final Map<String, String> connexionPropertiesMap = new HashMap<String, String>();
            connexionPropertiesMap.put(RmiInjectorService.RMIPORT, String.valueOf(rmiServer
                    .getRmiPort()));
            connexionPropertiesMap.put(RmiInjectorService.RMIIP, rmiServer.getHost());
            connexionPropertiesMap.put(RmiInjectorService.RMICONTEXT, rmiServer.getRmiContext());
            connexionPropertiesMap.put(RmiInjectorService.ACCEPTTIMEOUT,
                    RmiInjectorService.DEFAULTACCEPTTIMEOUT);
            return connexionPropertiesMap;
        } else {
            if (GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)) {
                Logger.getInstance().logErrorMessage("No RMI server available");
            }
            return null;
        }
    }

    /**
     * Allow to store temporary several attachements, for one injection
     * 
     * @param file
     * @param request
     * @return
     * @throws IOException
     * @throws URISyntaxException
     */
    public File storeAttachement(final IFileUploadPart file, final HttpServletRequest request)
            throws IOException, URISyntaxException {
        File repo = GeneralHelper.getRepositoryFile(request);
        File repoAttachement = new File(new URI(repo.toURI() + "/attachements"));
        if (!repoAttachement.exists()) {
            repoAttachement.mkdir();
        }
        File attachement = new File(new URI(repoAttachement.toURI() + "/" + file.getFileName()));
        if (!attachement.exists()) {
            attachement.createNewFile();
        }
        OutputStream out = null;
        InputStream in = null;
        try {
            out = new FileOutputStream(attachement);
            in = file.getFileInputStream();
            int readedBytes;
            /*
             * Write bytes into the the wsdl copy while readed data is available
             * (!= -1)
             */
            byte[] data = new byte[1024];
            do {
                readedBytes = in.read(data);
                if (readedBytes != -1) {
                    out.write(data, 0, readedBytes);
                    out.flush();
                }
            } while (readedBytes != -1);
        } finally {
            if (out != null) {
                out.close();
            }
            if (in != null) {
                in.close();
            }
        }
        return attachement;
    }

    public List<MessageExchange> getExchanges() {
        return exchanges;
    }

    public void setExchanges(List<MessageExchange> exchanges) {
        this.exchanges = exchanges;
    }

    public boolean checkRMIComponentAvailable(final ServerTO rmiServer)
            throws NumberFormatException, PetalsServiceException {
        final ManagementService service = (ManagementService) ManagementServiceFactory
                .getInstance().getService(rmiServer.getHost(),
                        Integer.parseInt(rmiServer.getJmxPort()), rmiServer.getJmxLogin(),
                        rmiServer.getJmxPassword());

        if (service != null) {
            final String componentStatus = service.getComponentState(RMI_COMPONENT_NAME);
            if (componentStatus != null && componentStatus.equals(ManagementService.STARTED)) {
                return true;
            }
        }
        return false;
    }

    public void removeAttachments(final List<String> deletedAttachments,
            final HttpServletRequest request) throws IOException, URISyntaxException {
        File repo = GeneralHelper.getRepositoryFile(request);
        File repoAttachement = new File(new URI(repo.toURI() + "/attachements"));
        if (repoAttachement.exists()) {
            for (final File file : repoAttachement.listFiles()) {
                if (deletedAttachments.contains(file.getName())) {
                    file.delete();
                }
            }
        }
    }

}
