/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.business;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.petals.tools.webconsole.util.ConfigHelper;
import org.ow2.petals.tools.webconsole.util.PetalsConsoleException;
import org.ow2.petals.tools.webconsole.xmlbeans.Server;
import org.xml.sax.SAXException;

/**
 * @author aruffie
 * 
 */
public class ServerFeatureBBean {
    public static final String ACTION_TYPE_MODIFY = "modify";

    public static final String ACTION_TYPE_ADD = "add";

    public static final String INSTANCE_FILE_NAME = "webconsole.xml";

    public static final String SCHEMA_FILE_NAME = "webconsole.xsd";

    private List<Server> configuredServers = null;

    public ServerFeatureBBean() throws PetalsConsoleException {
        populateServerList();
    }

    private List<Server> initializeConfiguredServers() throws MalformedURLException, IOException,
            SAXException, ParserConfigurationException, FactoryConfigurationError,
            URISyntaxException, PetalsConsoleException {
        return ConfigHelper.readConfiguration(ServerFeatureBBean.class.getClassLoader()
                .getResource(INSTANCE_FILE_NAME));
    }

    public void modifyServer(final String host, final String port, final String login,
            final String password, final String modifiedHost,final String modifiedJMXPort,
            final boolean installedRMIComponent, final int rmiPort, 
            final String rmiContext) throws PetalsConsoleException {

        for (final Server server : configuredServers) {
            if (server.getHost().equals(modifiedHost) && server.getJmxPort().equals(modifiedJMXPort)) {
                
                // Set General properties
                server.setHost(host);
                server.setJmxPort(port);
                server.setJmxPassword(password);
                server.setJmxLogin(login);
                
                // Set RMI properties
                server.setInstalledRmiComponent(installedRMIComponent);

                if(rmiContext != null && !rmiContext.equals("")){
                    server.setRmiContext(rmiContext);
                }
                if(rmiPort != 0){
                    server.setRmiPort(rmiPort);
                }
            }
        }

        try {
            final File file = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    INSTANCE_FILE_NAME).toURI().getPath());
            final File schema = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    SCHEMA_FILE_NAME).toURI().getPath());
            ConfigHelper.writeConfiguration(file, schema, this.configuredServers);
        } catch (URISyntaxException e) {
            throw new PetalsConsoleException(e);
        } catch (ParserConfigurationException e) {
            throw new PetalsConsoleException(e);
        } catch (IOException e) {
            throw new PetalsConsoleException(e);
        }
    }

    public void addServer(final String host, final String port, final String login,
            final String password, final boolean installedRMIComponent,
            final int rmiPort, final String rmiContext) throws PetalsConsoleException {
        for (final Server s : this.configuredServers) {
            if (s.getHost().equals(host) && s.getJmxPort().equals(port)
                    && s.getJmxPassword().equals(password) && s.getJmxLogin().equals(login)) {
                throw new PetalsConsoleException("Server configuration already exist in the "
                        + INSTANCE_FILE_NAME);
            }
        }

        final Server server = new Server();
        
        // Set General properties
        server.setHost(host);
        server.setJmxPort(port);
        server.setJmxPassword(password);
        server.setJmxLogin(login);
        
        // Set RMI properties
        if(rmiPort != 0 && (rmiContext != null && !rmiContext.equals(""))){
            server.setInstalledRmiComponent(installedRMIComponent);
        }
        if(rmiContext != null && !rmiContext.equals("")){
            server.setRmiContext(rmiContext);
        }
        if(rmiPort != 0){
            server.setRmiPort(rmiPort);
        }
        configuredServers.add(server);
        try {
            final File file = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    INSTANCE_FILE_NAME).toURI().getPath());
            final File schema = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    SCHEMA_FILE_NAME).toURI().getPath());
            ConfigHelper.writeConfiguration(file, schema, this.configuredServers);
        } catch (URISyntaxException e) {
            throw new PetalsConsoleException(e);
        } catch (ParserConfigurationException e) {
            throw new PetalsConsoleException(e);
        } catch (IOException e) {
            throw new PetalsConsoleException(e);
        }
    }

    public List<Server> getConfiguredServers() {
        return configuredServers;
    }

    public void setConfiguredServers(List<Server> configuredServers) {
        this.configuredServers = configuredServers;
    }

    private void populateServerList() throws PetalsConsoleException {
        try {
            this.configuredServers = initializeConfiguredServers();
        } catch (MalformedURLException e) {
            throw new PetalsConsoleException(e);
        } catch (IOException e) {
            throw new PetalsConsoleException(e);
        } catch (SAXException e) {
            throw new PetalsConsoleException(e);
        } catch (ParserConfigurationException e) {
            throw new PetalsConsoleException(e);
        } catch (FactoryConfigurationError e) {
            throw new PetalsConsoleException(e);
        } catch (URISyntaxException e) {
            throw new PetalsConsoleException(e);
        }

        if (this.configuredServers == null) {
            this.configuredServers = new ArrayList<Server>();
        }
    }

    public void deleteServer(final String host, final String port, final String login,
            final String password) throws PetalsConsoleException {

        int index = 0;
        for (final Server s : this.configuredServers) {
            if (s.getHost().equals(host) && s.getJmxPort().equals(port)
                    && s.getJmxPassword().equals(password) && s.getJmxLogin().equals(login)) {
                index = configuredServers.indexOf(s);
            }
        }
        configuredServers.remove(index);
        try {
            final File file = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    INSTANCE_FILE_NAME).toURI().getPath());
            final File schema = new File(ServerFeatureBBean.class.getClassLoader().getResource(
                    SCHEMA_FILE_NAME).toURI().getPath());
            ConfigHelper.writeConfiguration(file, schema, this.configuredServers);
        } catch (URISyntaxException e) {
            throw new PetalsConsoleException(e);
        } catch (ParserConfigurationException e) {
            throw new PetalsConsoleException(e);
        } catch (IOException e) {
            throw new PetalsConsoleException(e);
        }
    }
}
