/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.management.pooling;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.tools.webconsole.services.utils.ConnexionHelper;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.LogLevel;
import org.ow2.petals.tools.webconsole.util.Logger;

/**
 * @author aruffie
 */
public class CleaningWorker implements Runnable {

    private final long laptime;

    public CleaningWorker() {

        final Properties props = new Properties();

        // recover classloader

        /*
         * load general.properties file thank to classloader, in order to
         * recover timelap property.
         */
        try {
            props.load(this.getClass().getClassLoader().getResourceAsStream("general.properties"));
        } catch (final IOException e) {
            if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                Logger.getInstance().logThrowable(e);
            }
        }
        final String prop = props.getProperty("cleaningworker.laptime");

        if (prop != null) {
            this.laptime = Long.parseLong(prop);
        } else {
            this.laptime = 300000;
            if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                Logger.getInstance().logErrorMessage("Property 'cleaningworker.laptime' not found in general.propreties file," +
                		" Cleaning Worker has been initialized with 300000 for timelap");
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        while (true) {
            // Work each 'timelap'
            try {
                Thread.sleep(this.laptime);

                // perform cleaning into the client connections pool
                this.work();

            } catch (final InterruptedException e) {
                // Just log interruption
                if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                    Logger.getInstance().logThrowable(e);
                }
            }
        }
    }

    /**
     * Allow to clean unused client connections into the list
     */
    private void work() {
        final List<ClientContext> clientContextList = ConnexionHelper.getClientContextList();

        final List<ClientContext> clientRemoverList = new ArrayList<ClientContext>();

        // Check all unused connections
        for (final ClientContext clientContext : clientContextList) {
            if (!clientContext.isState()) {
                clientRemoverList.add(clientContext);
                try {
                    clientContext.getJmxClient().disconnect();
                } catch (final ConnectionErrorException e) {
                    if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                        Logger.getInstance().logThrowable(e);
                    }
                }
            } else {
                clientContext.setState(false);
            }
        }

        // Release all unused connections
        for (final ClientContext clientContext : clientRemoverList) {
            clientContextList.remove(clientContext);
            if(GeneralHelper.getLoggingLevel() != null){
                if(GeneralHelper.getLoggingLevel().equals(LogLevel.HIGH)){
                    Logger.getInstance().logInformation("Client connection '" + clientContext.getId()
                        + "' has been released from connections list");
                }
            } else {
                System.out.println("ERROR: Unable to recover the run-time value of the logging level.");
            }
        }

        clientRemoverList.clear();
    }
}
