/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.rmiinjector;

import java.io.IOException;
import java.util.Properties;

import org.ow2.petals.tools.webconsole.services.AbstractServiceFactory;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;

/**
 * @author aruffie
 * 
 */
public class RmiInjectorServiceFactory extends AbstractServiceFactory {
    private static RmiInjectorServiceFactory instance = null;

    private RmiInjectorService service = null;

    /**
     * function can allow to recover the factory instance
     */
    public static synchronized AbstractServiceFactory getInstance() {
        if (instance == null) {
            instance = new RmiInjectorServiceFactory();
        }
        return instance;
    }

    /**
     * Method allow to recover the proxyfication service from another layer
     * 
     * @throws PetalsServiceException
     */
    @Override
    public synchronized RmiInjectorService getService() throws PetalsServiceException {
        if (this.service == null) {
            this.service = this.getImplementation();
        }
        return this.service;
    }

    /**
     * Method allow to create the proxyfication service implementation
     */
    private synchronized RmiInjectorService getImplementation() throws PetalsServiceException {

        final Properties props = new Properties();

        // recover classloader

        // load services.properties file thank to classloader
        try {
            props.load(this.getClass().getClassLoader().getResourceAsStream("services.properties"));
        } catch (IOException e) {
            throw new PetalsServiceException(e);
        }
        final String className = props.getProperty("rmiinjector.service.impl");
        Class<?> loadedClass = null;
        try {
            loadedClass = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new PetalsServiceException(e);
        }
        boolean checkProxyficationServiceImplementation = Boolean.FALSE;

        Class<?>[] interfaces;
        interfaces = loadedClass.getInterfaces();
        for (final Class<?> inter : interfaces) {
            if (inter.equals(RmiInjectorService.class)) {
                checkProxyficationServiceImplementation = Boolean.TRUE;
            }
        }
        if (checkProxyficationServiceImplementation) {
            RmiInjectorService proxyficationService = null;
            try {
                proxyficationService = (RmiInjectorService) loadedClass.newInstance();
            } catch (InstantiationException e) {
                throw new PetalsServiceException(e);
            } catch (IllegalAccessException e) {
                throw new PetalsServiceException(e);
            }
            return proxyficationService;

        } else {
            throw new PetalsServiceException(loadedClass.getName() + " don't " + "implement "
                    + RmiInjectorService.class);
        }

    }
}
