/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.rmiinjector;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.MimetypesFileTypeMap;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.namespace.QName;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.tools.rmi.client.ComponentContextLocator;
import org.objectweb.petals.tools.rmi.common.util.InputStreamForker;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteComponentContext;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteDeliveryChannel;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.rmiinjector.model.Mep;
import org.ow2.petals.tools.webconsole.services.rmiinjector.model.Property;
import org.ow2.petals.tools.webconsole.services.rmiinjector.utils.StreamDataSource;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.LogLevel;
import org.ow2.petals.tools.webconsole.util.Logger;
import org.ow2.petals.tools.webconsole.util.StringHelper;

/**
 * @author aruffie
 * 
 */
public class RmiInjectorServiceImpl implements RmiInjectorService {

    private long defautAcceptTimeout;

    private RemoteComponentContext remoteComponentContext;

    private static final String UTF8 = "UTF-8";

    private int rmiPort = 1099;

    private String rmiContext = null;

    private String rmiIP = "localhost";

    private NormalizedMessage sendedNormalizedMessage;

    RmiInjectorServiceImpl() {
        this.defautAcceptTimeout = Long.parseLong(RmiInjectorService.DEFAULTACCEPTTIMEOUT);
        this.rmiContext = RmiInjectorService.DEFAULTRMICONTEXT;
        // System.setProperty("com.sun.xml.namespace.QName.useCompatibleSerialVersionUID",
        // "1.0");
    }

    public void initialize(final Map<String, String> propertiesMap) throws PetalsServiceException {
        /*
         * Properties checking is perform (empty, format ...)
         */
        if (propertiesMap != null) {
            if (propertiesMap.get(RMIPORT) != null) {
                try {
                    this.rmiPort = Integer.parseInt((String) propertiesMap.get(RMIPORT));
                } catch (NumberFormatException e) {
                    throw new PetalsServiceException("Bad format for the " + RMIPORT + " property",
                            e);
                }
            } else {
                throw new PetalsServiceException("property: " + RMIPORT
                        + ", wasn't present in the properties map");
            }
            if (propertiesMap.get(RMIIP) != null) {
                try {
                    this.rmiIP = (String) propertiesMap.get(RMIIP);
                } catch (NumberFormatException e) {
                    throw new PetalsServiceException("Bad format for the " + RMIIP + " property", e);
                }
            } else {
                throw new PetalsServiceException("property: " + RMIIP
                        + ", wasn't present in the properties map");
            }
            if (propertiesMap.get(RMICONTEXT) != null) {
                try {
                    this.rmiContext = (String) propertiesMap.get(RMICONTEXT);
                } catch (NumberFormatException e) {
                    throw new PetalsServiceException("Bad format for the " + RMICONTEXT
                            + " property", e);
                }
            } else {
                throw new PetalsServiceException("property: " + RMICONTEXT
                        + ", wasn't present in the properties map");
            }
            if (propertiesMap.get(ACCEPTTIMEOUT) != null) {
                try {
                    this.defautAcceptTimeout = Long.parseLong((String) propertiesMap
                            .get(ACCEPTTIMEOUT));
                } catch (NumberFormatException e) {
                    throw new PetalsServiceException("Bad format for the " + ACCEPTTIMEOUT
                            + " property", e);
                }
            } else {
                throw new PetalsServiceException("property: " + ACCEPTTIMEOUT
                        + ", wasn't present in the properties map");
            }
            ComponentContextLocator ccl;
            try {
                ccl = new ComponentContextLocator(rmiIP, rmiPort, rmiContext);
            } catch (final MalformedURLException e) {
                throw new PetalsServiceException(e);
            } catch (final RemoteException e) {
                throw new PetalsServiceException(
                        "Check your petals-se-rmi localized on '"
                                + rmiIP + "' server with RMI Port: "+rmiPort+" and RMI Context: "+rmiContext+". You can also remove or disabled rmi component declaration into 'webconsole.xml'", e);
            } catch (final NotBoundException e) {
                if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                    Logger.getInstance().logInformation(e.getMessage()+" not bound."+
                            " Check your petals-se-rmi localized on '"
                            + rmiIP + "' server with RMI Port: "+rmiPort+" and RMI Context: "+rmiContext+". You can also remove or disabled rmi component declaration into 'webconsole.xml'");
                }
                throw new PetalsServiceException(e.getMessage()+" not bound."+
                        " Check your petals-se-rmi localized on '"
                        + rmiIP + "' server with RMI Port: "+rmiPort+" and RMI Context: "+rmiContext+". You can also remove or disabled rmi component declaration into 'webconsole.xml'", e);
            }

            this.remoteComponentContext = ccl.getComponentContext();
        }        
    }

    public MessageExchange injectMessage(final QName _interface,
            final List<Property> normalizedProperties, final QName service, final String endpoint,
            final QName operation, final String mep, final List<Property> mEXProperties,
            final String content, final List<File> attachments, final Long synchronousSend)
            throws PetalsServiceException {

        Boolean timeoutReached = null;

        try {
            return sendSynch(mep, _interface, service, endpoint, operation, mEXProperties,
                    normalizedProperties, attachments, content, synchronousSend, timeoutReached);
        } catch (MessagingException e) {
            throw new PetalsServiceException(e);
        } catch (TransformerConfigurationException e) {
            throw new PetalsServiceException(e);
        } catch (TransformerException e) {
            throw new PetalsServiceException(e);
        } catch (IOException e) {
            throw new PetalsServiceException(e);
        }

    }

    private MessageExchange sendSynch(final String mep, final QName _interface,
            final QName service, final String endpoint, final QName operation,
            final List<Property> mEXProperties, final List<Property> normalizedProperties,
            final List<File> attachments, final String content, final Long synchronousSend,
            final Boolean timeoutReached) throws MessagingException, PetalsServiceException,
            TransformerConfigurationException, TransformerException, IOException {

        MessageExchange exchange = createMessageExchange(_interface, mep, service, endpoint,
                this.remoteComponentContext, operation, mEXProperties, normalizedProperties,
                attachments, content);
        if (synchronousSend != null) {
            this.sendedNormalizedMessage = exchange.getMessage("IN");
            /*
             * Fork the sended content to allow redisplay after the send
             */
            final InputStreamForker forker = StringHelper.fork(this.sendedNormalizedMessage
                    .getContent());

            /*
             * Reset the consomed source into the content, that will be send
             */
            this.sendedNormalizedMessage.setContent(new StreamSource(forker.getInputStreamOne()));
            if (this.remoteComponentContext == null) {
                throw new PetalsServiceException(
                        "The rmi remote component context is null, verify if it's correctly declared into the configuration file");
            }
            try {
                final RemoteDeliveryChannel channel = this.remoteComponentContext
                        .getDeliveryChannel();
                if (channel == null) {
                    throw new PetalsServiceException(
                            "Delivery channel of the rmi remote component context is unavailable ");
                } else {
                    exchange = channel.sendSync(exchange, synchronousSend);
                }
            } catch (final IllegalStateException e) {
                throw new PetalsServiceException(e);
            }

            if (exchange == null) {
                throw new PetalsServiceException("Injection failed, unexpected timeout reached, "
                        + "you have certainly assigned a timeout too short. Timeout used: "
                        + synchronousSend + " milli-seconds");
            }

            /*
             * Restore the consomed source into the content, after the send
             */
            this.sendedNormalizedMessage.setContent(new StreamSource(forker.getInputStreamTwo()));
            exchange.setMessage(this.sendedNormalizedMessage, "in");
        } else {
            this.sendedNormalizedMessage = exchange.getMessage("IN");

            /*
             * Fork the sended content to allow redisplay after the send
             */
            final InputStreamForker forker = StringHelper.fork(this.sendedNormalizedMessage
                    .getContent());

            /*
             * Reset the consomed source into the content, that will be send
             */
            this.sendedNormalizedMessage.setContent(new StreamSource(forker.getInputStreamOne()));
            this.remoteComponentContext.getDeliveryChannel().send(exchange);
            exchange = this.remoteComponentContext.getDeliveryChannel().accept(defautAcceptTimeout);

            /*
             * Restore the consomed source into the content, after the send
             */
            this.sendedNormalizedMessage.setContent(new StreamSource(forker.getInputStreamTwo()));
            exchange.setMessage(this.sendedNormalizedMessage, "in");
        }

        return exchange;
    }

    private static MessageExchange createMessageExchange(final QName interfaz, final String mep,
            final QName service, final String endpoint,
            final RemoteComponentContext componentContext, final QName operation,
            final List<Property> mEXProperties, final List<Property> normalizedProperties,
            final List<File> attachments, final String content) throws PetalsServiceException {
        MessageExchange msg = null;
        NormalizedMessage nm = null;

        try {
            if (Mep.IN_ONLY.value().equals(mep)) {
                msg = componentContext.getDeliveryChannel().createExchangeFactory()
                        .createInOnlyExchange();
                nm = msg.createMessage();
                ((InOnly) msg).setInMessage(nm);
            } else if (Mep.IN_OUT.value().equals(mep)) {

                msg = componentContext.getDeliveryChannel().createExchangeFactory()
                        .createInOutExchange();
                nm = msg.createMessage();
                ((InOut) msg).setInMessage(nm);
            } else if (Mep.IN_OPTIONAL_OUT.value().equals(mep)) {
                msg = componentContext.getDeliveryChannel().createExchangeFactory()
                        .createInOptionalOutExchange();
                nm = msg.createMessage();
                ((InOptionalOut) msg).setInMessage(nm);
            } else if (Mep.ROBUST_IN_ONLY.value().equals(mep)) {
                msg = componentContext.getDeliveryChannel().createExchangeFactory()
                        .createRobustInOnlyExchange();
                nm = msg.createMessage();
                ((RobustInOnly) msg).setInMessage(nm);
            }

            if (endpoint != null) {
                msg.setEndpoint(componentContext.getEndpoint(service, endpoint));
            }

            msg.setService(service);
            msg.setInterfaceName(interfaz);

            if (operation != null) {
                msg.setOperation(operation);
            }

            // setting global properties
            for (Property property : mEXProperties) {
                msg.setProperty(property.getName(), property.getValue());
            }

            // setting IN properties
            for (Property property : normalizedProperties) {
                nm.setProperty(property.getName(), property.getValue());
            }
        } catch (final MessagingException e) {
            throw new PetalsServiceException(e);
        } catch (final RemoteException e) {
            throw new PetalsServiceException(e);
        }
        // setting content
        try {
            nm.setContent(new StreamSource(new ByteArrayInputStream(content.getBytes(UTF8))));
        } catch (final UnsupportedEncodingException e) {
            if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                Logger.getInstance().logThrowable(e);
            }
        } catch (final MessagingException e) {
            throw new PetalsServiceException(e);
        }

        // setting attachments

        for (File attachment : attachments) {

            /*
             * set mimetype
             */
            final MimetypesFileTypeMap mime = new MimetypesFileTypeMap(RmiInjectorServiceImpl.class
                    .getClassLoader().getResourceAsStream("mimetypes.default"));

            /*
             * read file
             */
            InputStream in = null;
            try {
                in = new FileInputStream(attachment);
                byte[] data = new byte[in.available()];

                in.read(data);
                DataSource dataSource = new StreamDataSource(new ByteArrayInputStream(data),
                        attachment.getName(), mime.getContentType(attachment));
                nm.addAttachment(attachment.getName(), new DataHandler(dataSource));
            } catch (final IOException e) {
                throw new PetalsServiceException(e);
            } catch (final MessagingException e) {
                throw new PetalsServiceException(e);
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (final IOException e) {
                        throw new PetalsServiceException(e);
                    }
                }
            }
        }
        return msg;
    }

    /**
     * @return the remoteComponentContext
     */
    public RemoteComponentContext getRemoteComponentContext() {
        return remoteComponentContext;
    }
}
