/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.security.auth.login.FailedLoginException;

import org.ow2.petals.jmx.JMXClient;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.pooling.CleaningWorker;
import org.ow2.petals.tools.webconsole.services.management.pooling.ClientContext;

/**
 * @author aruffie
 * 
 */
public class ConnexionHelper {

    private final static List<ClientContext> CLIENTCONTEXTLIST;

    private final static Thread WORKER;

    /*
     * Initialize client connections pool, cleaning worker, and run the worker.
     */
    static {
        CLIENTCONTEXTLIST = Collections.synchronizedList(new ArrayList<ClientContext>());
        WORKER = new Thread(new CleaningWorker());
        WORKER.start();
    }

    private ConnexionHelper() {
        // Nop
    }

    public static JMXClient connect(final String hostname, final Integer jmxJNDIPort,
            final String jmxLogin, final String jmxPassword) throws PetalsServiceException {

        JMXClient jmxClient = checkPoolConnections(hostname, jmxJNDIPort, jmxLogin, jmxPassword);

        /*
         * If the associate jmxClient isn't cached, make one instance and store
         * its in the connections list
         */
        if (jmxClient == null) {
            try {
                jmxClient = new JMXClient(hostname, jmxJNDIPort, jmxLogin, jmxPassword);
                final ClientContext clientContext = new ClientContext(jmxClient, hostname + ":"
                        + jmxJNDIPort + ":" + jmxLogin + ":" + jmxPassword);
                CLIENTCONTEXTLIST.add(clientContext);
                return jmxClient;

            } catch (final ConnectionErrorException e) {
                String errMsg = "Can't connect to PEtALS server on " + hostname + ":" + jmxJNDIPort;
                if ((e.getCause() != null) && (e.getCause().getCause() != null)
                        && (e.getCause().getCause() instanceof FailedLoginException)) {
                    errMsg = errMsg + " - Invalid username or password";
                }
                throw new PetalsServiceException(errMsg, e);
            }
        }

        return jmxClient;
    }

    private static JMXClient checkPoolConnections(final String hostname, final Integer jmxJNDIPort,
            final String jmxLogin, final String jmxPassword) {

        // Create the id
        final StringBuilder builder = new StringBuilder();
        builder.append(hostname).append(":").append(jmxJNDIPort).append(":").append(jmxLogin)
                .append(":").append(jmxPassword);

        for (final ClientContext clientContext : CLIENTCONTEXTLIST) {
            if (clientContext.getId().equals(builder.toString())) {

                // Refresh the utilization state
                clientContext.setState(true);

                return clientContext.getJmxClient();
            }
        }

        return null;
    }

    /**
     * @return the clientcontextlist
     */
    public static List<ClientContext> getClientContextList() {
        return CLIENTCONTEXTLIST;
    }
}
