/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.utils;

import java.util.ArrayList;

import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.XmlElement;
import com.ebmwebsourcing.commons.jbi.sugenerator.cdk.BasicBuilderForCdkElements;
import com.ebmwebsourcing.commons.jbi.sugenerator.cdk.SuMode;

/**
 * @author aruffie
 * 
 */
public class SuGenerator {

    /**
     * Method allow you to create a SuBean Provider with the several parameters
     * provide
     * 
     * @param bc
     * @param consume
     * @param interfaceName
     * @param interfaceNameNsUrl
     * @param serviceName
     * @param serviceNameNsUrl
     * @param endpointName
     * @param linkType
     * @param suType
     * @param suForComponentName
     * @param suForComponentVersion
     * @param namespaces
     * @param cdkElements
     * @param componentelements
     * @return
     */
    public static SuBean createSuBeanProvide(String interfaceName, String interfaceNameNsUrl,
            String serviceName, String serviceNameNsUrl, String endpointName, String linkType,
            String suType, String suForComponentName, String suForComponentVersion,
            String namespaces, String operation, String timeout,
            ArrayList<XmlElement> componentElements, String wsdlFile) {
        SuBean suBean = new SuBean();

        // Basic data.
        suBean.setBc(true);
        suBean.setConsume(false);

        suBean.setInterfaceName(interfaceName);
        suBean.setInterfaceNamespaceUri(interfaceNameNsUrl);
        suBean.setServiceName(serviceName);
        suBean.setServiceNamespaceUri(serviceNameNsUrl);
        suBean.setEndpointName(endpointName);

        suBean.setLinkType(linkType);
        suBean.setSuType(suType);
        suBean.setComponentName(suForComponentName);
        suBean.setComponentName(suForComponentVersion);

        // Name spaces.
        suBean.addNamespace("soap", "http://petals.ow2.org/components/soap/version-3.1");
        suBean.addNamespace("petalsCDK", "http://petals.ow2.org/components/extensions/version-4.0");

        // Complex data.
        ArrayList<XmlElement> cdkElements = BasicBuilderForCdkElements.getCdkElements30(
                SuMode.provide, wsdlFile, timeout, operation, null);
        suBean.cdkElements.addAll(cdkElements);

        suBean.specificElements.addAll(componentElements);
        return suBean;
    }

    /**
     * Method allow you to create a SuBean Consumer with the several parameters
     * 
     * @param interfaceName
     * @param interfaceNameNsUrl
     * @param serviceName
     * @param serviceNameNsUrl
     * @param endpointName
     * @param linkType
     * @param suType
     * @param suForComponentName
     * @param suForComponentVersion
     * @param namespaces
     * @param operation
     * @param timeout
     * @param componentElements
     * @return
     */
    public static SuBean createSuBeanConsume(String interfaceName, String interfaceNameNsUrl,
            String serviceName, String serviceNameNsUrl, String endpointName, String linkType,
            String suType, String suForComponentName, String suForComponentVersion,
            String namespaces, String operation, String timeout,
            ArrayList<XmlElement> componentElements) {
        SuBean suBean = new SuBean();

        // Basic data.
        suBean.setBc(true);
        suBean.setConsume(true);

        suBean.setInterfaceName(interfaceName);
        suBean.setInterfaceNamespaceUri(interfaceNameNsUrl);
        suBean.setServiceName(serviceName);
        suBean.setServiceNamespaceUri(serviceNameNsUrl);
        suBean.setEndpointName(endpointName);

        suBean.setLinkType(linkType);
        suBean.setSuType(suType);
        suBean.setComponentName(suForComponentName);
        suBean.setComponentVersion(suForComponentVersion);

        // Name spaces.

        suBean.addNamespace("soap", "http://petals.ow2.org/components/soap/version-3.1");
        suBean.addNamespace("petalsCDK", "http://petals.ow2.org/components/extensions/version-4.0");

        // Complex data.
        ArrayList<XmlElement> cdkElements = BasicBuilderForCdkElements.getCdkElements30(
                SuMode.consume, null, timeout, operation, "InOut");
        suBean.cdkElements.addAll(cdkElements);

        suBean.specificElements.addAll(componentElements);
        return suBean;
    }
}
