/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.petals.tools.webconsole.business.InstallServiceAssemblyBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_SERVICE_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;

/**
 * @author aruffie
 * 
 */
public class InstallServiceAssemblyUIBean {

    private static final String LOCAL_INSTALLATION = "local";

    private static final String URL_INSTALLATION = "url";

    private URL fileURL;

    private InstallServiceAssemblyBBean businessBean;

    private String serverName;

    private String url;

    private String visibility;

    private List<String> visibilities;
    
    private boolean urlVisibility;
    
    private boolean localVisibility;

    public InstallServiceAssemblyUIBean() {
        this.visibilities = loadVisibilities();
        this.visibility = LOCAL_INSTALLATION;
        this.localVisibility = true;
        
        this.businessBean = new InstallServiceAssemblyBBean();
    }

    private List<String> loadVisibilities() {
        final List<String> list = new ArrayList<String>();

        list.add(LOCAL_INSTALLATION);
        list.add(URL_INSTALLATION);

        return list;
    }

    public String getVisibility() {
        return visibility;
    }

    public void setVisibility(String visibility) {
        this.visibility = visibility;
    }

    public List<String> getVisibilities() {
        return visibilities;
    }

    public void setVisibilities(List<String> visibilities) {
        this.visibilities = visibilities;
    }

    public boolean getBrowserVisibility() {

        if (visibility.equals(LOCAL_INSTALLATION)) {
            return true;
        }
        return false;
    }

    public boolean getUrlVisibility() {

        if (visibility.equals(URL_INSTALLATION)) {
            return true;
        }
        return false;
    }

    public void upload(final IFileUploadPart file, final String serverName,
            final String domainName, final HttpServletRequest request) throws NonLocalizedError {
        try {
            this.fileURL = businessBean.storeServiceAssembly(file, request);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        }
        this.serverName = serverName;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String install(final HttpServletRequest request, final String serverName)
            throws NonLocalizedError {
        this.serverName = serverName;
        ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute(
                "managementUIBean");
        if (visibility.equals(URL_INSTALLATION)) {
            try {
                businessBean.beginInstall(request, managementUIBean, this.url, this.serverName);
            } catch (final MalformedURLException e) {
                final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR, e
                        .getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            } catch (final PetalsServiceException e) {
                throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
            } catch (final IOException e) {
                throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
            } catch (final URISyntaxException e) {
                final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                        .getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            }
        } else if (visibility.equals(LOCAL_INSTALLATION)) {

            if (this.fileURL != null) {
                try {
                    businessBean.beginInstall(request, managementUIBean, this.fileURL.toString(),
                            this.serverName);
                } catch (final MalformedURLException e) {
                    final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR, e
                            .getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final PetalsServiceException e) {
                    throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
                } catch (final URISyntaxException e) {
                    final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                            .getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                }
            }
        }
        managementUIBean.reconnect(request);
        return "success";
    }
    
    public void getCheckRadioButton(){
        if(visibility.equals(LOCAL_INSTALLATION))
        {
            this.localVisibility = true;
            this.urlVisibility = false;
        }
        else if(visibility.equals(URL_INSTALLATION))
        {
            this.urlVisibility = true;
            this.localVisibility = false;
        }
    }
    
    /**
     * @return the localVisibility
     */
    public boolean isLocalVisibility() {
        return localVisibility;
    }

    /**
     * @param localVisibility the localVisibility to set
     */
    public void setLocalVisibility(boolean localVisibility) {
        this.localVisibility = localVisibility;
    }

    /**
     * @param urlVisibility the urlVisibility to set
     */
    public void setUrlVisibility(boolean urlVisibility) {
        this.urlVisibility = urlVisibility;
    }
    
    /**
     * @return the urlVisibility
     */
    public boolean isUrlVisibility() {
        return urlVisibility;
    }
}
