/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.NUMBER_FORMAT_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.WSDLEXCEPTION_ERROR;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.petals.tools.webconsole.business.InstallServiceAssemblyBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.services.proxyfication.ProxyficationService;
import org.ow2.petals.tools.webconsole.services.proxyfication.ProxyficationServiceFactory;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_SERVICE_ERROR;

/**
 * @author aruffie
 * 
 */
public class ProxyServiceUIBean {

    private static final String DEFAULT_TIMEOUT = "30";

    private static final String DEFAULT_PROXYNAME_BEGIN = "sa-";

    private static final String DEFAULT_PROXYNAME_ENDING = "-Proxy";

    private static final String DEFAULT_PROVIDERNAME_BEGIN = "su-";

    private static final String DEFAULT_PROVIDERNAME_ENDING = "-Provide";

    private static final String DEFAULT_PROVIDERADDR = "Proxy";

    private static final String DEFAULT_RESTCONSUMERSUNAME_BEGIN = "su-";

    private static final String DEFAULT_RESTCONSUMERSUNAME_ENDING = "RESTProxyConsume";

    private static final String DEFAULT_RESTCONSUMERSUADDR = "RESTProxy";

    private static final String DEFAULT_SOAPCONSUMERSUNAME_BEGIN = "su-";

    private static final String DEFAULT_SOAPCONSUMERSUNAME_ENDING = "SOAPProxyConsume";

    private static final String DEFAULT_SOAPCONSUMERSUADDR = "SOAPProxy";

    private Service selectedService;

    private File wsdlProvided;

    /*
     * Protocol informations
     */
    private boolean rest;

    private boolean soap;

    /*
     * General informations
     */
    private String proxyName;

    private String timeout;

    /*
     * For external service informations
     */
    private String suProviderName;

    private String suProviderAddr;

    /*
     * REST informations
     */
    private String RESTConsumerSUName;

    private String RESTConsumerSUAddr;

    private String RESTNameSpaceURI;

    private String RESTNameSpacePrefix;

    private String RESTNSRemovedPrefix;

    /*
     * SOAP informations
     */
    private String SOAPConsumerSUName;

    private String SOAPConsumerSUAddr;

    private List<ServerTO> servers;

    private ServerTO selectedServer;

    private String selectedServerName;

    public void load(final HttpServletRequest request) {
        final AddWSDLUIBean addWSDLBean = (AddWSDLUIBean) request.getSession().getAttribute(
                "addWSDLUIBean");
        this.selectedService = addWSDLBean.getSelectedService();
        this.wsdlProvided = addWSDLBean.getWsdlProvided();
        this.timeout = DEFAULT_TIMEOUT;

        /*
         * ProxyName affectation
         */
        this.proxyName = DEFAULT_PROXYNAME_BEGIN + this.selectedService.getQName().getLocalPart()
                + DEFAULT_PROXYNAME_ENDING;

        /*
         * for external service
         */
        if (!addWSDLBean.isInternalService()) {
            /*
             * Provide address affectation
             */
            this.suProviderName = DEFAULT_PROVIDERNAME_BEGIN
                    + this.selectedService.getQName().getLocalPart() + DEFAULT_PROVIDERNAME_ENDING;
            this.suProviderAddr = this.selectedService.getQName().getLocalPart()
                    + DEFAULT_PROVIDERADDR;
        }

        /*
         * Load REST informations
         */
        this.RESTConsumerSUName = DEFAULT_RESTCONSUMERSUNAME_BEGIN
                + this.selectedService.getQName().getLocalPart()
                + DEFAULT_RESTCONSUMERSUNAME_ENDING;
        this.RESTConsumerSUAddr = this.selectedService.getQName().getLocalPart()
                + DEFAULT_RESTCONSUMERSUADDR;

        /*
         * Load SOAP informations
         */
        this.SOAPConsumerSUName = DEFAULT_SOAPCONSUMERSUNAME_BEGIN
                + this.selectedService.getQName().getLocalPart()
                + DEFAULT_SOAPCONSUMERSUNAME_ENDING;
        this.SOAPConsumerSUAddr = this.selectedService.getQName().getLocalPart()
                + DEFAULT_SOAPCONSUMERSUADDR;

        /*
         * Load all servers
         */
        this.servers = ((ManagementUIBean) request.getSession().getAttribute("managementUIBean"))
                .getDomain().getServers();
    }

    /**
     * Add Proxy after submit of the form
     * 
     * @return String
     * @throws NonLocalizedError
     */
    public String submitCreateProxy(final HttpServletRequest request) throws NonLocalizedError {

        final AddWSDLUIBean addWSDLBean = (AddWSDLUIBean) request.getSession().getAttribute(
                "addWSDLUIBean");
        /*
         * Get the proxyfication service
         */
        ProxyficationService proxyficationService = null;
        try {
            proxyficationService = (ProxyficationService) ProxyficationServiceFactory.getInstance()
                    .getService();
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
        }
        /*
         * Modify the wsdl for the provider into PEtALS
         */
        URI uriNewWSDL;
        try {
            uriNewWSDL = new URI(addWSDLBean.getWsdlProvided().toURI() + "Proxy");
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        File modifiedWSDL = null;
        try {
            modifiedWSDL = proxyficationService.modifySoapAddressWSDL(
                    addWSDLBean.getWsdlProvided(), addWSDLBean.getWsdlProvided().getParentFile(),
                    new File(uriNewWSDL), this.suProviderAddr);
        } catch (final WSDLException e) {
            throw new NonLocalizedError(WSDLEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        File proxySA = null;
        try {
            /*
             * create proxy by service
             */
            proxySA = proxyficationService.addProxy(this.soap, this.rest, this.selectedService
                    .getQName().getLocalPart(), this.SOAPConsumerSUAddr, this.suProviderAddr,
                    addWSDLBean.getWsdlVersion(), this.selectedService.getInterface().getQName()
                            .getLocalPart(), this.selectedService.getInterface().getQName()
                            .getNamespaceURI(), this.timeout, this.suProviderAddr,
                    this.selectedService.getQName().getNamespaceURI(), this.SOAPConsumerSUAddr,
                    this.RESTConsumerSUAddr, this.RESTNameSpacePrefix, this.RESTNameSpaceURI,
                    this.RESTNSRemovedPrefix, addWSDLBean.getWsdlProvided(), this.proxyName,
                    addWSDLBean.getWsdlProvided().getParentFile(), this.RESTConsumerSUName,
                    this.SOAPConsumerSUName, modifiedWSDL);
        } catch (final WSDLException e) {
            throw new NonLocalizedError(WSDLEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
        }

        ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute(
                "managementUIBean");
        InstallServiceAssemblyBBean businessBean = new InstallServiceAssemblyBBean();
        URL proxySAURL = null;
        try {
            proxySAURL = businessBean.storeServiceAssembly(proxySA, request);
            businessBean.beginInstall(request, managementUIBean, proxySAURL.toString(),
                    this.selectedServerName);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
        }

        managementUIBean.reconnect(request);

        try {
            final File file = new File(new URI(GeneralHelper.getRepositoryFile(request).toURI()
                    + "/service-assemblies/" + proxySA.getName()));
            if (file != null && file.exists()) {
                file.delete();
            }
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        }

        try {
            final File wsdlTree = new File(new URI(GeneralHelper.getRepositoryFile(request).toURI()
                    + "/wsdl"));
            if (wsdlTree != null && wsdlTree.exists()) {
                GeneralHelper.deleteDirectoryContent(wsdlTree);
            }
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        }

        return "success";
    }

    public void addProxy(final File proxySAFile, final HttpServletRequest request)
            throws NonLocalizedError {

        /*
         * get the management service
         */
        ManagementService service = null;
        try {
            service = (ManagementService) ManagementServiceFactory.getInstance().getService(
                    this.selectedServer.getHost(),
                    Integer.parseInt(this.selectedServer.getJmxPort()),
                    this.selectedServer.getJmxLogin(), this.selectedServer.getJmxPassword());
        } catch (final NumberFormatException e) {
            final NonLocalizedError error = new NonLocalizedError(NUMBER_FORMAT_ERROR,
                    "The port value: " + this.selectedServer.getJmxPort()
                            + " is not properly formatted " + e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
        }

        /*
         * install the proxy service assembly
         */
        try {
            service.installServiceAssembly(proxySAFile.toURL());
            GeneralHelper.deleteUploadedArtifact(request, proxySAFile.toURL().toString());
        } catch (final MalformedURLException e) {
            final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR, e.getLocalizedMessage(), e);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        File repo = null;
        try {
            repo = GeneralHelper.getRepositoryFile(request);
            File repoAttachement = new File(new URI(repo.toURI() + "/wsdl"));
            GeneralHelper.deleteDirectoryContent(repoAttachement);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR, e.getLocalizedMessage(), e);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

    }

    public Service getSelectedService() {
        return this.selectedService;
    }

    public void setSelectedService(Service selectedService) {
        this.selectedService = selectedService;
    }

    public File getWsdlProvided() {
        return this.wsdlProvided;
    }

    public void setWsdlProvided(File wsdlProvided) {
        this.wsdlProvided = wsdlProvided;
    }

    public boolean isRest() {
        return this.rest;
    }

    public void setRest(boolean rest) {
        this.rest = rest;
    }

    public boolean isSoap() {
        return this.soap;
    }

    public void setSoap(boolean soap) {
        this.soap = soap;
    }

    public String getProxyName() {
        return this.proxyName;
    }

    public void setProxyName(String proxyName) {
        this.proxyName = proxyName;
    }

    public String getTimeout() {
        return this.timeout;
    }

    public void setTimeout(String timeout) {
        this.timeout = timeout;
    }

    public String getSuProviderName() {
        return this.suProviderName;
    }

    public void setSuProviderName(String suProviderName) {
        this.suProviderName = suProviderName;
    }

    public String getSuProviderAddr() {
        return this.suProviderAddr;
    }

    public void setSuProviderAddr(String suProviderAddr) {
        this.suProviderAddr = suProviderAddr;
    }

    public String getRESTConsumerSUName() {
        return this.RESTConsumerSUName;
    }

    public void setRESTConsumerSUName(String consumerSUName) {
        if ((consumerSUName != null) && (!consumerSUName.equals(""))) {
            this.RESTConsumerSUName = consumerSUName;
        }
    }

    public String getRESTConsumerSUAddr() {
        return this.RESTConsumerSUAddr;
    }

    public void setRESTConsumerSUAddr(String consumerSUAddr) {
        if ((consumerSUAddr != null) && (!consumerSUAddr.equals(""))) {
            this.RESTConsumerSUAddr = consumerSUAddr;
        }
    }

    public String getRESTNameSpaceURI() {
        return this.RESTNameSpaceURI;
    }

    public void setRESTNameSpaceURI(String nameSpaceURI) {
        this.RESTNameSpaceURI = nameSpaceURI;
    }

    public String getRESTNameSpacePrefix() {
        return this.RESTNameSpacePrefix;
    }

    public void setRESTNameSpacePrefix(String nameSpacePrefix) {
        this.RESTNameSpacePrefix = nameSpacePrefix;
    }

    public String getRESTNSRemovedPrefix() {
        return this.RESTNSRemovedPrefix;
    }

    public void setRESTNSRemovedPrefix(String removedPrefix) {
        this.RESTNSRemovedPrefix = removedPrefix;
    }

    public String getSOAPConsumerSUName() {
        return this.SOAPConsumerSUName;
    }

    public void setSOAPConsumerSUName(String consumerSUName) {
        if ((consumerSUName != null) && (!consumerSUName.equals(""))) {
            this.SOAPConsumerSUName = consumerSUName;
        }
    }

    public String getSOAPConsumerSUAddr() {
        return this.SOAPConsumerSUAddr;
    }

    public void setSOAPConsumerSUAddr(String consumerSUAddr) {
        if ((consumerSUAddr != null) && (!consumerSUAddr.equals(""))) {
            this.SOAPConsumerSUAddr = consumerSUAddr;
        }
    }

    public List<ServerTO> getServers() {
        return this.servers;
    }

    public void setServers(List<ServerTO> servers) {
        this.servers = servers;
    }

    public ServerTO getSelectedServer() {
        return this.selectedServer;
    }

    public void setSelectedServer(ServerTO selectedServer) {
        this.selectedServer = selectedServer;
    }

    public String getSelectedServerName() {
        return this.selectedServerName;
    }

    public void setSelectedServerName(String selectedServerName) {
        this.selectedServerName = selectedServerName;
    }

}
