/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.petals.tools.webconsole.business.ServerFeatureBBean;
import org.ow2.petals.tools.webconsole.util.PetalsConsoleException;
import org.ow2.petals.tools.webconsole.xmlbeans.Server;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_CONSOLE_ERROR;

/**
 * @author aruffie
 * 
 */
public class ServerFeatureUIBean {

    private boolean writeServer;

    private String host = "localhost";

    private String port = "7700";

    private String login = "petals";

    private String password = "petals";

    private ServerFeatureBBean businessBean = null;

    private String modifiedHost = "localhost";
    
    private String modifiedJMXPort;

    private String rmiContext;
    
    private int rmiPort;
    
    private boolean installedRMIComponent;
    
    public ServerFeatureUIBean() throws NonLocalizedError {
        try {
            businessBean = new ServerFeatureBBean();
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR, e.getLocalizedMessage(), e);
        }
    }

    public boolean isWriteServer() {
        return writeServer;
    }

    public void load(final HttpServletRequest request) throws NonLocalizedError {
        
        
        if(request.getParameter("selectedHost") == null || request.getParameter("selectedHost").equals("")){
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR, "Hostname unavailable for selected server");
        }
        if(request.getParameter("selectedJMXPort") == null || request.getParameter("selectedJMXPort").equals("")){
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR, "JMX port unavailable for selected server");
        }
        final String selectedHost = request.getParameter("selectedHost");
        final String selectedJMXPort = request.getParameter("selectedJMXPort");
        for (final Server server : businessBean.getConfiguredServers()) {
            if (server.getHost().equals(selectedHost) && server.getJmxPort().equals(selectedJMXPort)) {
                this.host = server.getHost();
                this.port = server.getJmxPort();
                this.login = server.getJmxLogin();
                this.password = server.getJmxPassword();
                this.modifiedHost = server.getHost();
                this.modifiedJMXPort = server.getJmxPort();
                this.installedRMIComponent = server.isInstalledRmiComponent();
                this.rmiContext = server.getRmiContext();
                this.rmiPort = server.getRmiPort();
            }
        }
    }

    public void setWriteServer(boolean writeServer) {
        this.writeServer = writeServer;
    }

    public void saveServer(final String host, final String port, final String login,
            final String password) throws NonLocalizedError {
        try {
            businessBean.addServer(host, port, login, password, false, 0, null);
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR, e.getLocalizedMessage(), e);
        }
    }

    public String performServerForm(final HttpServletRequest request) throws NonLocalizedError {

        final String actionType = request.getParameter("actionType");

        if (actionType.equals(ServerFeatureBBean.ACTION_TYPE_MODIFY)) {
            try {
                businessBean.modifyServer(this.host, this.port, this.login, this.password,
                        this.modifiedHost, this.modifiedJMXPort, this.installedRMIComponent, this.rmiPort,
                        this.rmiContext);
            } catch (final PetalsConsoleException e) {
                throw new NonLocalizedError(PETALS_CONSOLE_ERROR, e.getLocalizedMessage(), e);
            }
        } else if (actionType.equals(ServerFeatureBBean.ACTION_TYPE_ADD)) {
            try {
                businessBean.addServer(this.host, this.port, this.login, this.password,
                        this.installedRMIComponent, this.rmiPort, this.rmiContext);
            } catch (final PetalsConsoleException e) {
                throw new NonLocalizedError(PETALS_CONSOLE_ERROR, e.getLocalizedMessage(), e);
            }
        }
        final ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession()
                .getAttribute("managementUIBean");
        if (managementUIBean != null) {
            managementUIBean.setHost(host);
            managementUIBean.setPort(port);
            managementUIBean.setLogin(login);
            managementUIBean.setPassword(password);
            managementUIBean.updateDomain();
        }
        return "success";
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getPort() {
        return port;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public List<Server> getConfiguredServers() {
        return businessBean.getConfiguredServers();
    }

    public void deleteServer(final HttpServletRequest request) throws NonLocalizedError {
        final String host = request.getParameter("host");
        final String port = request.getParameter("port");
        final String login = request.getParameter("login");
        final String password = request.getParameter("password");
        try {
            businessBean.deleteServer(host, port, login, password);
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR, e.getLocalizedMessage(), e);
        }
    }

    /**
     * @return the rmiContext
     */
    public String getRmiContext() {
        return rmiContext;
    }

    /**
     * @param rmiContext the rmiContext to set
     */
    public void setRmiContext(String rmiContext) {
        this.rmiContext = rmiContext;
    }

    /**
     * @return the rmiPort
     */
    public int getRmiPort() {
        return rmiPort;
    }

    /**
     * @param rmiPort the rmiPort to set
     */
    public void setRmiPort(int rmiPort) {
        this.rmiPort = rmiPort;
    }

    /**
     * @return the installedRMIComponent
     */
    public boolean isInstalledRMIComponent() {
        return installedRMIComponent;
    }

    /**
     * @param installedRMIComponent the installedRMIComponent to set
     */
    public void setInstalledRMIComponent(boolean installedRMIComponent) {
        this.installedRMIComponent = installedRMIComponent;
    }
    
    
}
