/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.petals.tools.webconsole.business.ManagementBBean;
import org.ow2.petals.tools.webconsole.business.ServerBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.to.ComponentTO;
import org.ow2.petals.tools.webconsole.to.DomainTO;
import org.ow2.petals.tools.webconsole.to.EndPointTO;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.to.ServiceAssemblyTO;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.LogLevel;
import org.ow2.petals.tools.webconsole.util.Logger;
import org.ow2.petals.tools.webconsole.util.StringHelper;

public class ServerUIBean {

    private String serverName;

    private boolean monitoringStarted;

    private String[] selectedComponents;

    private String[] selectedSharedLibraries;

    private String[] selectedServiceAssemblies;

    private List<ComponentTO> bindingcomponents;

    private List<ComponentTO> serviceEngines;

    private List<String> sharedLibraries;

    private List<ServiceAssemblyTO> serviceAssemblies;

    private List<EndPointTO> endpointsList;

    private List<String> selectedEndpointsList;

    private ManagementUIBean managementBean;

    private ServerBBean businessBean;

    private static final String RECOVERY_SERVICE_ERROR = "Recovery service error";

    private static final String DOWNLOAD_WSDL_ERROR = "Download wsdl error";

    private static final String START_COMPONENT_ACTION_ERROR = "Error(s) occured during start component action";

    private static final String STOP_COMPONENT_ACTION_ERROR = "Error(s) occured during stop component action";

    private static final String SHUTDOWN_COMPONENT_ACTION_ERROR = "Error(s) occured during shutdown component action";

    private static final String UNINSTALL_COMPONENT_ACTION_ERROR = "Error(s) occured during uninstall component action";

    private static final String START_SA_ACTION_ERROR = "Error(s) occured during start service assembly action";

    private static final String STOP_SA_ACTION_ERROR = "Error(s) occured during stop service assembly action";

    private static final String SHUTDOWN_SA_ACTION_ERROR = "Error(s) occured during shutdown service assembly action";

    private static final String UNINSTALL_SA_ACTION_ERROR = "Error(s) occured during uninstall service assembly action";

    private static final String UNINSTALL_SL_ACTION_ERROR = "Error(s) occured during uninstall shared library action";

    private static final String FORCE_UNLOAD_COMPONENT_ERROR = "Error(s) occured during force unload component action";

    private static final String FORCE_UNDEPLOY_SERVICE_ASSEMBLY_ERROR = "Error(s) occured during force undeploy service assembly action";

    // ====================================================================
    // === Control Methods
    // ====================================================================
    public void load(HttpServletRequest iRequest) throws NonLocalizedError {
        this.serverName = iRequest.getParameter("serverName");
        this.businessBean = new ServerBBean();
        managementBean = (ManagementUIBean) iRequest.getSession().getAttribute("managementUIBean");
        // Check if the configuration has been modified

        if (managementBean != null) {
            managementBean.connect();
            for (ServerTO server : managementBean.getDomain().getServers()) {
                if (server.getName().equals(serverName)) {
                    this.bindingcomponents = server.getBindingComponents();
                    this.serviceEngines = server.getServiceEngines();
                    this.serviceAssemblies = server.getServiceAssemblies();
                    this.sharedLibraries = server.getSharedLibraries();
                    this.serverName = server.getName();
                }
            }
        }
        if (this.bindingcomponents == null) {
            this.bindingcomponents = new ArrayList<ComponentTO>();
        }
        if (this.serviceEngines == null) {
            this.serviceEngines = new ArrayList<ComponentTO>();
        }
        if (this.serviceAssemblies == null) {
            this.serviceAssemblies = new ArrayList<ServiceAssemblyTO>();
        }
    }

    /**
     * Allow to start a selected component
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void startSelectedComponent(final HttpServletRequest request) throws NonLocalizedError {
        final String componentName = request.getParameter("componentName");
        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.startComponent(componentName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(START_COMPONENT_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to start several components
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String startSelectedComponents(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service;
        service = serviceRecovery(request);

        /*
         * For all selected components start these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedComponents != null) {
            for (final String componentName : selectedComponents) {
                try {
                    service.startComponent(componentName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedComponents = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(START_COMPONENT_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to shutdown a selected component
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void shutdownSelectedComponent(final HttpServletRequest request)
            throws NonLocalizedError {
        final String componentName = request.getParameter("componentName");
        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.shutdownComponent(componentName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(SHUTDOWN_COMPONENT_ACTION_ERROR,
                    e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to shutdown several selected components
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String shutdownSelectedComponents(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected components shutdown these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedComponents != null) {
            for (final String componentName : selectedComponents) {
                try {
                    service.shutdownComponent(componentName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedComponents = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(SHUTDOWN_COMPONENT_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to stop a selected component
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void stopSelectedComponent(final HttpServletRequest request) throws NonLocalizedError {
        final String componentName = request.getParameter("componentName");
        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * stop this component
         */
        try {
            service.stopComponent(componentName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(STOP_COMPONENT_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to stop several selected components
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String stopSelectedComponents(final HttpServletRequest request) throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected components stop these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedComponents != null) {
            for (final String componentName : selectedComponents) {
                try {
                    service.stopComponent(componentName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }
        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedComponents = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(STOP_COMPONENT_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to uninstall a selected component
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void uninstallSelectedComponent(final HttpServletRequest request)
            throws NonLocalizedError {
        final String componentName = request.getParameter("componentName");
        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * uninstall this component
         */
        try {
            service.uninstallComponent(componentName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_COMPONENT_ACTION_ERROR,
                    e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to uninstall several selected components
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String uninstallSelectedComponents(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected components stop these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedComponents != null) {
            for (final String componentName : selectedComponents) {
                try {
                    service.uninstallComponent(componentName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        /*
         * Finally update the domain view
         */

        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_COMPONENT_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to uninstall a selected shared library
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void uninstallSelectedLibrary(final HttpServletRequest request) throws NonLocalizedError {
        final String libraryName = request.getParameter("libraryName");
        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * uninstall shared library
         */
        try {
            service.uninstallSharedLibrary(libraryName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_SL_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */

        resetServerUIBean();
        managementBean.reconnect(request);
        updateModification();
    }

    /**
     * Allow to uninstall selected libraries
     * 
     * @param request
     * @return
     * @throws NonLocalizedError
     */
    public String uninstallSelectedLibraries(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected shared libraries uninstall these
         */
        final List<String> errors = new ArrayList<String>();
        if (selectedSharedLibraries != null) {
            for (final String slName : selectedSharedLibraries) {
                try {
                    service.uninstallSharedLibrary(slName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        /*
         * Finally update the domain view
         */

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        this.selectedSharedLibraries = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_SL_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to start the selected service assembly
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void startSelectedServiceAssembly(final HttpServletRequest request)
            throws NonLocalizedError {

        final String saName = request.getParameter("saName");
        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.startServiceAssembly(saName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(START_SA_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);

    }

    /**
     * Allow to start selected service assemblies
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String startSelectedServiceAssemblies(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected service assemblies start these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedServiceAssemblies != null) {
            for (final String saName : selectedServiceAssemblies) {
                try {
                    service.startServiceAssembly(saName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedServiceAssemblies = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(START_SA_ACTION_ERROR, builder
                    .toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    private void updateRMIInjectorUIBean(final HttpServletRequest request) {
        RMIInjectorUIBean rmiInjectorUIBean = (RMIInjectorUIBean) request.getSession()
                .getAttribute("rmiInjectorUIBean");

        if (rmiInjectorUIBean != null) {
            rmiInjectorUIBean.getOperations().clear();

            rmiInjectorUIBean.getEndpoints().clear();
            rmiInjectorUIBean.getEndpointsNames().clear();

            for (final EndPointTO endpoint : this.endpointsList) {
                rmiInjectorUIBean.getEndpoints().add(endpoint);
                rmiInjectorUIBean.getEndpointsNames().add(endpoint.getName());
            }
        } else {
            if(GeneralHelper.getLoggingLevel().equals(LogLevel.MEDIUM)){
                Logger.getInstance().logErrorMessage("Unable to update RMI Injector");
            }
        }
    }

    /**
     * Allow to stop selected service assembly
     * 
     * @param HttpServletREquest
     *            request
     * @throws NonLocalizedError
     */
    public void stopSelectedServiceAssembly(final HttpServletRequest request)
            throws NonLocalizedError {

        final String saName = request.getParameter("saName");
        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.stopServiceAssembly(saName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(STOP_SA_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to stop all selected service assemblies
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String stopSelectedServiceAssemblies(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected service assemblies stop these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedServiceAssemblies != null) {
            for (final String saName : selectedServiceAssemblies) {
                try {
                    service.stopServiceAssembly(saName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedServiceAssemblies = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(STOP_SA_ACTION_ERROR, builder
                    .toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to uninstall selected service assemblies
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String uninstallSelectedServiceAssemblies(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected service assemblies uninstall these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedServiceAssemblies != null) {
            for (final String saName : selectedServiceAssemblies) {
                try {
                    service.uninstallServiceAssembly(saName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_SA_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to uninstall the selected service assembly
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void uninstallSelectedServiceAssembly(final HttpServletRequest request)
            throws NonLocalizedError {

        final String saName = request.getParameter("saName");

        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.uninstallServiceAssembly(saName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_SA_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedServiceAssemblies = null;
    }

    /**
     * Allow to shutdown the selected service assembly
     * 
     * @param HttpServletRequest
     *            request
     * @throws NonLocalizedError
     */
    public void shutdownSelectedServiceAssembly(final HttpServletRequest request)
            throws NonLocalizedError {

        final String saName = request.getParameter("saName");
        ManagementService service = null;
        service = serviceRecovery(request);

        try {
            service.shutdownServiceAssembly(saName);
        } catch (PetalsServiceException e) {
            final NonLocalizedError error = new NonLocalizedError(SHUTDOWN_SA_ACTION_ERROR, e
                    .getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
    }

    /**
     * Allow to shutdown all selected service assemblies
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String shutdownSelectedServiceAssemblies(final HttpServletRequest request)
            throws NonLocalizedError {

        ManagementService service = null;
        service = serviceRecovery(request);

        /*
         * For all selected service assemblies shutdown these
         */
        final List<String> errors = new ArrayList<String>();
        if (this.selectedServiceAssemblies != null) {
            for (final String saName : selectedServiceAssemblies) {
                try {
                    service.shutdownServiceAssembly(saName);
                } catch (PetalsServiceException e) {
                    errors.add(e.getLocalizedMessage());
                }
            }
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedServiceAssemblies = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(UNINSTALL_SA_ACTION_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    // ##############################################################
    // Getter & Setter
    // ##############################################################

    public String getServerName() {
        return serverName;
    }

    public void setServerName(String serverName) {
        this.serverName = serverName;
    }

    public boolean isMonitoringStarted() {
        return monitoringStarted;
    }

    public void setMonitoringStarted(boolean monitoringStarted) {
        this.monitoringStarted = monitoringStarted;
    }

    public String[] getSelectedComponents() {
        return selectedComponents;
    }

    public void setSelectedComponents(String[] selectedComponents) {
        this.selectedComponents = selectedComponents;
    }

    public String[] getSelectedSharedLibraries() {
        return selectedSharedLibraries;
    }

    public void setSelectedSharedLibraries(String[] selectedSharedLibraries) {
        this.selectedSharedLibraries = selectedSharedLibraries;
    }

    public String[] getSelectedServiceAssemblies() {
        return selectedServiceAssemblies;
    }

    public void setSelectedServiceAssemblies(String[] selectedServiceAssemblies) {
        this.selectedServiceAssemblies = selectedServiceAssemblies;
    }

    public List<ComponentTO> getBindingcomponents() {
        return bindingcomponents;
    }

    public void setBindingcomponents(List<ComponentTO> bindingcomponents) {
        this.bindingcomponents = bindingcomponents;
    }

    public List<ComponentTO> getServiceEngines() {
        return serviceEngines;
    }

    public void setServiceEngines(List<ComponentTO> serviceEngines) {
        this.serviceEngines = serviceEngines;
    }

    public List<String> getSharedLibraries() {
        return sharedLibraries;
    }

    public void setSharedLibraries(List<String> sharedLibraries) {
        this.sharedLibraries = sharedLibraries;
    }

    public List<ServiceAssemblyTO> getServiceAssemblies() {
        return serviceAssemblies;
    }

    public void setServiceAssemblies(List<ServiceAssemblyTO> serviceAssemblies) {
        this.serviceAssemblies = serviceAssemblies;
    }

    public List<ComponentTO> getComponents() {
        List<ComponentTO> allComponents = new ArrayList<ComponentTO>();

        if (this.bindingcomponents != null && this.bindingcomponents.size() > 0) {
            allComponents.addAll(this.bindingcomponents);
        }

        if (this.serviceEngines != null && this.serviceEngines.size() > 0) {
            allComponents.addAll(this.serviceEngines);
        }

        return allComponents;
    }

    public List<EndPointTO> getEndpointsList() {
        this.endpointsList = new ArrayList<EndPointTO>();
        for (ComponentTO componentTO : this.bindingcomponents) {
            for (EndPointTO endpointTO : componentTO.getEndpoints()) {
                endpointsList.add(endpointTO);
            }
        }
        for (ComponentTO componentTO : this.serviceEngines) {
            for (EndPointTO endpointTO : componentTO.getEndpoints()) {
                endpointsList.add(endpointTO);
            }
        }
        return endpointsList;
    }

    public List<String> getSelectedEndpointsList() {
        return selectedEndpointsList;
    }

    public void setSelectedEndpointsList(List<String> selectedEndpointsList) {
        this.selectedEndpointsList = selectedEndpointsList;
    }

    public void setEndpointsList(List<EndPointTO> endpointsList) {
        this.endpointsList = endpointsList;
    }

    /**
     * Allow to recover the management service for all internal functions in
     * this class
     * 
     * @return ManagementService
     * @throws NonLocalizedError
     */
    private ManagementService serviceRecovery(final HttpServletRequest request)
            throws NonLocalizedError {

        /*
         * Get connection parameters for the current selected server
         */
        if (StringHelper.isNullOrEmpty(this.serverName)) {
            this.serverName = request.getParameter("serverName");
        }

        managementBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
        if (managementBean != null) {
            final ServerTO server = managementBean.getDomain().findServer(this.serverName);
            if (server != null) {
                managementBean.setHost(server.getHost());
                managementBean.setLogin(server.getJmxLogin());
                managementBean.setPassword(server.getJmxPassword());
                managementBean.setPort(server.getJmxPort());
            }

            ManagementService service = null;
            try {
                service = (ManagementService) ManagementServiceFactory.getInstance().getService(
                        managementBean.getHost(), Integer.parseInt(managementBean.getPort()),
                        managementBean.getLogin(), managementBean.getPassword());
            } catch (NumberFormatException e) {
                throw new NonLocalizedError(RECOVERY_SERVICE_ERROR, e.getLocalizedMessage(), e);
            } catch (PetalsServiceException e) {
                throw new NonLocalizedError(RECOVERY_SERVICE_ERROR, e.getLocalizedMessage(), e);
            }
            return service;
        } else {
            throw new NonLocalizedError("Error management connection",
                    "Management connection lost for the server: " + this.serverName);
        }
    }

    private void updateModification() throws NonLocalizedError {
        for (ServerTO server : managementBean.getDomain().getServers()) {
            if (server.getName().equals(serverName)) {
                this.bindingcomponents = server.getBindingComponents();
                this.serviceEngines = server.getServiceEngines();
                this.serviceAssemblies = server.getServiceAssemblies();
                this.endpointsList = server.getAllEndpoints();
                this.sharedLibraries = server.getSharedLibraries();
                this.serverName = server.getName();
            }
        }
    }

    private void resetServerUIBean() {
        this.bindingcomponents.clear();
        this.serviceEngines.clear();
        this.serviceAssemblies.clear();
        this.endpointsList.clear();
        this.sharedLibraries.clear();
    }

    /**
     * Allow to recover the wsdl linked to the selected endpoint choosen by the
     * user
     * 
     * @param request
     * @param response
     * @throws NonLocalizedError
     */
    public void downloadWsdl(final HttpServletRequest request, final HttpServletResponse response)
            throws NonLocalizedError {
        final String endpointName = request.getParameter("endpointName");
        String serviceName = null;

        /*
         * Find service name linked to the endpointName
         */
        for (final EndPointTO endpoint : endpointsList) {
            if (endpoint.getName().equals(endpointName)) {
                serviceName = endpoint.getService();
            }
        }
        String prettyWsdl = null;
        try {
            prettyWsdl = this.businessBean.loadWsdlEndpoint(serviceName, endpointName,
                    managementBean.getHost(), Integer.parseInt(managementBean.getPort()),
                    managementBean.getLogin(), managementBean.getPassword());
        } catch (NumberFormatException e) {
            throw new NonLocalizedError(DOWNLOAD_WSDL_ERROR, e.getLocalizedMessage(), e);
        } catch (PetalsServiceException e) {
            throw new NonLocalizedError(DOWNLOAD_WSDL_ERROR, e.getLocalizedMessage(), e);
        }
        response.setContentType("text/xml");
        ServletOutputStream out = null;
        try {
            out = response.getOutputStream();
            out.write(prettyWsdl.getBytes());
            out.flush();
        } catch (final IOException e) {
            throw new NonLocalizedError(DOWNLOAD_WSDL_ERROR, e.getLocalizedMessage(), e);
        } finally {
            prettyWsdl = null;
            this.businessBean = null;
            try {
                out.close();
            } catch (final IOException e) {
                throw new NonLocalizedError(DOWNLOAD_WSDL_ERROR, e.getLocalizedMessage(), e);
            }
        }
    }

    public void updateServer(final DomainTO domain) {
        final ServerTO currentServer = domain.findServer(this.serverName);
        if (currentServer != null) {

            /*
             * Reset and add all elements
             */
            this.bindingcomponents = currentServer.getBindingComponents();
            this.serviceAssemblies = currentServer.getServiceAssemblies();
            this.sharedLibraries = currentServer.getSharedLibraries();
            this.serviceEngines = currentServer.getServiceEngines();
            this.endpointsList = currentServer.getAllEndpoints();
        }
    }

    /**
     * Allow to force unload all selected components
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String forceUnloadSelectedComponents(final HttpServletRequest request)
            throws NonLocalizedError {

        final ManagementBBean businessBean = new ManagementBBean();
        ManagementService service = null;
        service = serviceRecovery(request);
        /*
         * For all selected components force unload these
         */
        final List<String> errors = new ArrayList<String>();
        for (final String component : this.selectedComponents) {
            try {
                businessBean.forceUnload(component, service);
            } catch (final PetalsServiceException e) {
                errors.add(e.getLocalizedMessage());
            }
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        this.managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedComponents = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(FORCE_UNLOAD_COMPONENT_ERROR,
                    builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to force unload all selected service assemblies
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String forceUnDeploySelectedServiceAssemblies(final HttpServletRequest request)
            throws NonLocalizedError {

        final ManagementBBean businessBean = new ManagementBBean();
        ManagementService service = null;
        service = serviceRecovery(request);
        /*
         * For all selected components force unload these
         */
        final List<String> errors = new ArrayList<String>();
        for (final String sa : this.selectedServiceAssemblies) {
            try {
                businessBean.forceUnDeploy(sa, service);
            } catch (final PetalsServiceException e) {
                errors.add(e.getLocalizedMessage());
            }
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        this.selectedServiceAssemblies = null;
        if (errors.isEmpty()) {
            return "success";
        } else {
            final StringBuilder builder = new StringBuilder();

            for (final String error : errors) {
                builder.append(error + "\n\n");
            }
            final NonLocalizedError error = new NonLocalizedError(
                    FORCE_UNDEPLOY_SERVICE_ASSEMBLY_ERROR, builder.toString());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    /**
     * Allow to force unload selected component
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String forceUnloadSelectedComponent(final HttpServletRequest request)
            throws NonLocalizedError {

        final ManagementBBean businessBean = new ManagementBBean();
        ManagementService service = null;
        service = serviceRecovery(request);

        final String componentName = request.getParameter("componentName");

        /*
         * force unload specifique component
         */
        try {
            businessBean.forceUnload(componentName, service);
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(FORCE_UNLOAD_COMPONENT_ERROR, e.getLocalizedMessage(), e);
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        this.managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        return "success";
    }

    /**
     * Allow to force unload selected service assembly
     * 
     * @param HttpServletRequest
     *            request
     * @return String
     * @throws NonLocalizedError
     */
    public String forceUnDeploySelectedServiceAssembly(final HttpServletRequest request)
            throws NonLocalizedError {

        final ManagementBBean businessBean = new ManagementBBean();
        ManagementService service = null;
        service = serviceRecovery(request);

        final String saName = request.getParameter("saName");

        /*
         * force unload specifique service assembly
         */
        try {
            businessBean.forceUnDeploy(saName, service);
        } catch (PetalsServiceException e) {
            throw new NonLocalizedError(FORCE_UNDEPLOY_SERVICE_ASSEMBLY_ERROR, e
                    .getLocalizedMessage(), e);
        }

        /*
         * Finally update the domain view
         */
        resetServerUIBean();
        managementBean.reconnect(request);

        updateModification();
        updateRMIInjectorUIBean(request);
        return "success";
    }
}
