/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.util;

/**
 * @author aruffie
 * This class represent the petals webconsole
 * Logger
 */
public class Logger{
    
    /*
     * Log Webconsole information pattern
     */
    private final static String INFORMATION = "INFO: ";
    
    /*
     * Log Webconsole error pattern
     */
    private final static String ERROR = "ERROR: ";
    
    private static Logger instance;
    
    private Logger(){
        // NOP
    }
    
    /**
     * Allow to log information linked to webconsole behaviour
     * @param information <String> refering to occured
     * information
     */
    public void logInformation(final String information){
        System.out.println(INFORMATION+information);
    }
    
    /**
     * Allow to log error linked to webconsole behaviour
     * @param error <String> refering to occured
     * error 
     */
    public void logErrorMessage(final String error){
        System.out.println(ERROR+error);
    }
    
    /**
     * Allow to log error linked to webconsole behaviour
     * @param throwable <Throwable> refering to occured
     * throwable
     */
    public void logThrowable(final Throwable throwable){
        throwable.printStackTrace();
    }
    
    /**
     * Get singleton instance of the logger
     * @return Return the unique Logger instance
     */
    public static synchronized Logger getInstance(){
        if(instance == null){
            instance = new Logger();
        }
        return instance;
    }
}
