/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.util;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Node;

/**
 * XML Pretty printer utility class
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 1.0
 * 
 */
public class XMLPrettyPrinter {
    private static final String prettyPrintStylesheet = "<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' version='1.0' "
            + " xmlns:xalan='http://xml.apache.org/xslt' "
            + " exclude-result-prefixes='xalan'>"
            + "  <xsl:output method='xml' indent='yes' omit-xml-declaration='yes' xalan:indent-amount='4'/>"
            + "  <xsl:strip-space elements='*'/>"
            + "  <xsl:template match='/'>"
            + "    <xsl:apply-templates/>"
            + "  </xsl:template>"
            + "  <xsl:template match='node() | @*'>"
            + "        <xsl:copy>"
            + "          <xsl:apply-templates select='node() | @*'/>"
            + "        </xsl:copy>"
            + "  </xsl:template>" + "</xsl:stylesheet>";

    /**
     * Prettify the node into the output stream.
     * 
     * @param node
     * @param out
     * @throws Exception
     */
    public static void prettify(Node node, OutputStream out) throws Exception {
        Source source = new DOMSource(node);
        Source stylesheetSource = new StreamSource(new ByteArrayInputStream(prettyPrintStylesheet
                .getBytes()));

        TransformerFactory tFactory = TransformerFactory.newInstance();
        Templates templates = tFactory.newTemplates(stylesheetSource);
        Transformer transformer = templates.newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.transform(source, new StreamResult(out));
    }
}
